"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _iterall = require("iterall");

var _graphqlSubscriptions = require("graphql-subscriptions");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// $FlowFixMe
class KafkaPubSubIterator {
  constructor(pubsub, eventNames) {
    _defineProperty(this, "allSubscribed", void 0);

    _defineProperty(this, "eventsArray", void 0);

    _defineProperty(this, "listening", void 0);

    _defineProperty(this, "pubsub", void 0);

    _defineProperty(this, "pullQueue", void 0);

    _defineProperty(this, "pushQueue", void 0);

    this.pubsub = pubsub;
    this.pullQueue = [];
    this.pushQueue = [];
    this.listening = true;
    this.eventsArray = Array.isArray(eventNames) ? eventNames : [eventNames];
    this.allSubscribed = this.subscribeAll();
  }

  emptyQueue(subscriptionIds) {
    if (this.listening) {
      this.listening = false;
      this.unsubscribeAll(subscriptionIds);
      this.pullQueue.forEach(resolve => resolve({
        value: undefined,
        done: true
      }));
      this.pullQueue.length = 0;
      this.pushQueue.length = 0;
    }
  }

  async next() {
    await this.allSubscribed;
    return this.listening ? this.pullValue() : this.return();
  }

  pullValue() {
    return new Promise(resolve => {
      if (this.pushQueue.length !== 0) {
        resolve({
          value: this.pushQueue.shift(),
          done: false
        });
      } else {
        this.pullQueue.push(resolve);
      }
    });
  }

  async pushValue(message) {
    await this.allSubscribed;

    if (this.pullQueue.length !== 0) {
      this.pullQueue.shift()({
        value: message,
        done: false
      });
    } else {
      this.pushQueue.push(message);
    }
  }

  async return() {
    this.emptyQueue((await this.allSubscribed));
    return {
      value: undefined,
      done: true
    };
  }

  subscribeAll() {
    return Promise.all(this.eventsArray.map(eventName => this.pubsub.subscribe(eventName, this.pushValue.bind(this))));
  }

  async throw(error) {
    this.emptyQueue((await this.allSubscribed));
    return Promise.reject(error);
  }

  unsubscribeAll(subscriptionIds) {
    subscriptionIds.forEach(subscriptionId => this.pubsub.unsubscribe(subscriptionId));
  } // $FlowFixMe


  [_iterall.$$asyncIterator]() {
    return this;
  }

}

var _default = KafkaPubSubIterator;
exports.default = _default;