"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _graphqlSubscriptions = require("graphql-subscriptions");

var _kafkaNode = require("kafka-node");

var _v = _interopRequireDefault(require("uuid/v4"));

var _kafkaPubsubIterator = _interopRequireDefault(require("./kafka-pubsub-iterator"));

var _kafkaConsumer = _interopRequireDefault(require("./kafka/kafka-consumer"));

var _kafkaProducer = _interopRequireDefault(require("./kafka/kafka-producer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class KafkaPubSub {
  // {[channel: string]: Array<number>};
  constructor(options) {
    _defineProperty(this, "channelSubscriptions", void 0);

    _defineProperty(this, "consumer", void 0);

    _defineProperty(this, "options", void 0);

    _defineProperty(this, "producer", void 0);

    _defineProperty(this, "subIdCounter", void 0);

    _defineProperty(this, "subscriptionMap", void 0);

    this.options = options;
    this.subscriptionMap = {};
    this.channelSubscriptions = {};
    this.subIdCounter = 0;
    this.consumer = (0, _kafkaConsumer.default)({
      host: this.options.host,
      topicName: this.options.topic,
      groupName: (0, _v.default)(),
      // to get message for all node
      processMessage: message => {
        const parsedMessage = JSON.parse(message.value.toString());

        if (parsedMessage.channel) {
          const {
            channel,
            ...payload
          } = parsedMessage;
          this.onMessage(parsedMessage.channel, payload); // No channel abstraction, publish over the whole topic
        } else {
          this.onMessage(this.options.topic, parsedMessage);
        }
      }
    });
    this.producer = new _kafkaProducer.default();
  }

  asyncIterator(triggers) {
    return new _kafkaPubsubIterator.default(this, triggers);
  }

  onMessage(channel, message) {
    const subscriptions = this.channelSubscriptions[channel];

    if (subscriptions) {
      subscriptions.forEach(subId => {
        const [, listener] = this.subscriptionMap[subId];
        listener(message);
      });
    } // no subscribers, don't publish msg

  }

  async publish(payload) {
    return this.producer.produce(this.options.topic, payload);
  }

  subscribe(channel, onMessage) {
    this.subIdCounter++;
    this.subscriptionMap[this.subIdCounter] = [channel, onMessage];
    this.channelSubscriptions[channel] = [...(this.channelSubscriptions[channel] || []), this.subIdCounter];
    return Promise.resolve(this.subIdCounter);
  }

  unsubscribe(index) {
    const [channel] = this.subscriptionMap[index];
    this.channelSubscriptions[channel] = this.channelSubscriptions[channel].filter(subId => subId !== index);
    delete this.subscriptionMap[index];
  }

}

var _default = KafkaPubSub;
exports.default = _default;