"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _kafkaNode = require("kafka-node");

var _logger = require("../logger");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class KafkaProducer {
  constructor(id = 'kafka-producer') {
    _defineProperty(this, "producer", null);

    _defineProperty(this, "id", '');

    _defineProperty(this, "produce", async (topic, payloads) => {
      const producer = await this.getProducer();
      return new Promise((resolve, reject) => {
        producer.send([{
          topic,
          messages: JSON.stringify(payloads)
        }], (err, data) => {
          if (err) reject(err);
          resolve(data);
        });
      });
    });

    _defineProperty(this, "getProducer", async () => new Promise((resolve, reject) => {
      if (this.producer) {
        resolve(this.producer);
      } else {
        _logger.logger.info('Ty to connect to %s.', process.env.KAFKA_HOST);

        const client = new _kafkaNode.KafkaClient({
          kafkaHost: String(process.env.KAFKA_HOST)
        });
        const producerOptions = {
          // Configuration for when to consider a message as acknowledged, default 1
          requireAc1ks: 1,
          // The amount of time in milliseconds to wait for all acks before considered, default 100ms
          ackTimeoutMs: 100
        };
        const newProducer = new _kafkaNode.HighLevelProducer(client, producerOptions);
        newProducer.on('error', error => {
          _logger.logger.error('Producer [%s] not created: %o.', this.id, error);

          reject(error);
        });
        newProducer.on('ready', () => {
          _logger.logger.info('Producer [%s] created.', this.id);

          this.producer = newProducer;
          resolve(newProducer);
        });
      }
    }));

    this.id = id;
  }

}

var _default = KafkaProducer;
exports.default = _default;